@@ -19,8 +19,8 @@ from __future__ import print_function, absolute_import
 from pkg_resources import get_distribution, DistributionNotFound
 
 # For backwards compatibility
-from sshreader.ssh import SSH, shell_command, do_shell_script
-from sshreader.sshreader import ServerJob, Hook, sshread, print_results
+from sshreader.ssh import SSH, shell_command, do_shell_script, envvars
+from sshreader.sshreader import ServerJob, Hook, sshread, print_results, echo
 
 __author__ = 'Jesse Almanrode (jesse@almanrode.com)'
 try:
