@@ -18,36 +18,69 @@ shell_command function for running local shell scripts!
 #     You should have received a copy of the GNU Lesser General Public License
 #     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 from __future__ import print_function
+import logging
 import os
 import paramiko
-import logging
 import warnings
-from subprocess import Popen, PIPE, STDOUT
 from collections import namedtuple
+from getpass import getuser
+from subprocess import Popen, PIPE, STDOUT
 
 __author__ = 'Jesse Almanrode (jesse@almanrode.com)'
 
 # Using namedtuple because... why not?
 ShellCommand = namedtuple('ShellCommand', ['cmd', 'stdout', 'stderr', 'return_code'])
 ShellCommandCombined = namedtuple('ShellCommandCombined', ['cmd', 'stdout', 'return_code'])
+EnvVars = namedtuple('EnvVars', ['username', 'rsa_key', 'dsa_key'])
 
 
-def shell_command(command, combine=False):
+def envvars():
+    """ Attempt to determine the current username and location of any ssh keys.  If any value is unable to be determined
+    it is returned as 'None'.
+
+    :return: NamedTuple of (username, rsa_key, dsa_key)
+    """
+    user = None
+    rsa_key = None
+    dsa_key = None
+    if os.getlogin() == getuser():
+        user = getuser()
+    userhome = os.path.expanduser('~')
+    if os.path.exists(userhome + "/.ssh"):
+        keyfiles = os.listdir(userhome + "/.ssh")
+        if "id_rsa" in keyfiles:
+            rsa_key = userhome + "/.ssh/id_rsa"
+        if "id_dsa" in keyfiles:
+            dsa_key = userhome + "/.ssh/id_dsa"
+    return EnvVars(user, rsa_key, dsa_key)
+
+
+def shell_command(command, combine=False, decodebytes=False):
     """Run a command in the shell on localhost and return the output
 
     :param command: String containing the shell script to run
-    :param combine: Direct stderr to stdout (combine output)
+    :param combine: Direct stderr to stdout (default = False)
+    :param decodebytes: Decode bytes objects to unicode strings (default = False)
     :return: NamedTuple for (cmd, stdout, stderr) or (cmd, stdout)
     """
     if combine:
         pipeout = Popen(command, shell=True, stdout=PIPE, stderr=STDOUT)
         stdout, stderr = pipeout.communicate()
         assert stderr is None
-        result = ShellCommandCombined(cmd=command, stdout=stdout.strip(), return_code=pipeout.returncode)
+        if decodebytes:
+            result = ShellCommandCombined(cmd=command, stdout=stdout.decode().strip(),
+                                          return_code=pipeout.returncode)
+        else:
+            result = ShellCommandCombined(cmd=command, stdout=stdout.strip(), return_code=pipeout.returncode)
     else:
         pipeout = Popen(command, shell=True, stdout=PIPE, stderr=PIPE)
         stdout, stderr = pipeout.communicate()
-        result = ShellCommand(cmd=command, stdout=stdout.strip(), stderr=stderr.strip(), return_code=pipeout.returncode)
+        if decodebytes:
+            result = ShellCommand(cmd=command, stdout=stdout.decode().strip(),
+                                  stderr=stderr.decode().strip(), return_code=pipeout.returncode)
+        else:
+            result = ShellCommand(cmd=command, stdout=stdout.strip(), stderr=stderr.strip(),
+                                  return_code=pipeout.returncode)
     return result
 
 
@@ -73,7 +106,7 @@ class SSH(object):
     :param timeout: SSH connection timeout in seconds (default = 30)
     :param connect: Initiate the connect (default = True)
     :return: SSH connection object
-    :raises: BadAuthenticationType
+    :raises: SSHException
     """
     def __init__(self, fqdn, username=None, password=None, keyfile=None, port=22, timeout=30, connect=True):
         if keyfile is None and username is None:
@@ -92,14 +125,23 @@ class SSH(object):
         if connect:
             self.connect()
 
-    def ssh_command(self, command, timeout=30, combine=False):
+    def __enter__(self):
+        if self.is_alive() is False:
+            self.connect()
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.close()
+
+    def ssh_command(self, command, timeout=30, combine=False, decodebytes=False):
         """Run a command over an ssh connection
 
         :param command: The command to run
         :param timeout: Timeout for the command
         :param combine: Combine stderr and stdout
+        :param decodebytes: Decode bytes objects to unicode strings (default = False)
         :return: Namedtuple of (cmd, stdout, stderr, return_code) or (cmd, stdout, return_code)
-        :raises SSHException:
+        :raises: SSHException
         """
         if self.is_alive() is False:
             raise paramiko.SSHException("Connection is not established")
@@ -112,11 +154,20 @@ class SSH(object):
             chan.exec_command(command)
             stdout = stdout_file.read()
             stdout_file.close()
-            result = ShellCommandCombined(cmd=command, stdout=stdout.strip(), return_code=chan.recv_exit_status())
+            if decodebytes:
+                result = ShellCommandCombined(cmd=command, stdout=stdout.decode().strip(),
+                                              return_code=chan.recv_exit_status())
+            else:
+                result = ShellCommandCombined(cmd=command, stdout=stdout.strip(), return_code=chan.recv_exit_status())
         else:
             stdin, stdout, stderr = self.connection.exec_command(command, timeout=timeout)
-            result = ShellCommand(cmd=command, stdout=stdout.read().strip(), stderr=stderr.read().strip(),
-                                  return_code=stdout.channel.recv_exit_status())
+            if decodebytes:
+                result = ShellCommand(cmd=command, stdout=stdout.read().decode().strip(),
+                                      stderr=stderr.read().decode().strip(),
+                                      return_code=stdout.channel.recv_exit_status())
+            else:
+                result = ShellCommand(cmd=command, stdout=stdout.read().strip(), stderr=stderr.read().strip(),
+                                      return_code=stdout.channel.recv_exit_status())
         return result
 
     def close(self):
@@ -150,7 +201,7 @@ class SSH(object):
         """Opens an SSH Connection
 
         :return: True
-        :raises SSHException"
+        :raises: SSHException
         """
         logging.basicConfig()  # http://stackoverflow.com/questions/26659772/
         if self.is_alive():
@@ -168,4 +219,4 @@ class SSH(object):
             else:
                 self.connection.connect(self.host, port=self.port, username=self.username,
                                         password=self.password, timeout=self.timeout, look_for_keys=False)
-        return True
+        return True
\ No newline at end of file
